﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Windows.Forms;

namespace PressureSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		PressureSensor openCh = null;
		formCleaner cleaner;
		PressureSensor baro = null;
		private ErrorEventBox errorBox;
		private DataGraphBox pressureGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(PressureSensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			baro = openCh ?? openArgs.makeChannel<PressureSensor>();
			pressureGraphBox = new DataGraphBox(this, "Pressure");

			this.Size = this.MinimumSize;

			baro.Attach += baro_attach;
			baro.Detach += baro_detach;
			baro.Error += baro_error;
			baro.PressureChange += baro_change;
			baro.PropertyChange += baro_propertychange;

			try {
				baro.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			baro.Attach -= baro_attach;
			baro.Detach -= baro_detach;
			baro.Error -= baro_error;
			baro.PressureChange -= baro_change;
			baro.PropertyChange -= baro_propertychange;
			baro.Close();
		}

		void baro_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			PressureSensor attachedDevice = (PressureSensor)sender;

			pressureGraphBox.SetTitle(DataGraphBox.GenerateTitle("Pressure", (Phidget)attachedDevice));

			try {
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.Value = attachedDevice.DataInterval;

				changeTriggerTrk.Unit = "kPa";
				changeTriggerTrk.minTxt = attachedDevice.MinPressureChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinPressureChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxPressureChangeTrigger;
				changeTriggerTrk.maxTxt = attachedDevice.MaxPressureChangeTrigger.ToString();
				changeTriggerTrk.Value = attachedDevice.PressureChangeTrigger;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}
		void baro_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}
		void baro_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				pressureGraphBox.addGap();
		}

		private void baro_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = baro.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "PressureChangeTrigger":
				changeTriggerTrk.Value = baro.PressureChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = baro.DataRate;
					updateGraphDataInterval();
				}
				break;
			}
		}

		void baro_change(object sender, Phidget22.Events.PressureSensorPressureChangeEventArgs e) {
			try {
				barovalue.Text = e.Pressure.ToString() + " kPa";
				pressureGraphBox.addData(e.Pressure, baro.DataInterval);
			} catch (PhidgetException ex) { errorBox.addMessage("Error reading pressure: " + ex.Message); }
		}
		private void updateGraphDataInterval() {
			try {
				if (baro.PressureChangeTrigger == 0)
					pressureGraphBox.DataInterval = (1000.0 / baro.DataRate);
				else
					pressureGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}

		private void pressureGraphButton_Click(object sender, EventArgs e) {
			if (!pressureGraphBox.Visible)
				pressureGraphBox.Show();
		}
		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temp = dataIntervalTrk.Value;
						baro.DataInterval = (int)temp;
						if (baro.DataInterval < (int)temp) {
							while (baro.DataInterval != (int)temp) {
								temp++;
								baro.DataInterval = (int)temp;
							}
						} else {
							while (baro.DataInterval != (int)temp) {
								temp--;
								baro.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = baro.DataInterval;
						dataIntervalTrk.Value = baro.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						baro.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			} else //changetrigger control
			  {
				try {
					baro.PressureChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = baro.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = baro.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (baro.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = baro.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = baro.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}
